/*
 *   ORXONOX - the hottest 3D action shooter ever to exist
 *                    > www.orxonox.net <
 *
 *
 *   License notice:
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public License
 *   as published by the Free Software Foundation; either version 2
 *   of the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 *   Author:
 *      Noah Zarro
 *      Theo von Arx
 *   Co-authors:
 *      
 *
 */

/**
    @file SOBGumbaBoss.cc
    @brief All items in this minigame inherit from this class. Items can move around like platforms and enemies.
*/

#include "SOB.h"
#include "SOBGumba.h"
#include "SOBGumbaBoss.h"
#include "SOBFireball.h"
#include "SOBFigure.h"

#include "core/CoreIncludes.h"
#include "core/XMLPort.h"
#include "util/Output.h"
#include <BulletCollision/NarrowPhaseCollision/btManifoldPoint.h>



namespace orxonox
{
    RegisterClass(SOBGumbaBoss);

    SOBGumbaBoss::SOBGumbaBoss(Context* context) : SOBGumba(context)
    {
        RegisterObject(SOBGumbaBoss);

        gumbaMaxTime_ = 4;
        gumbaTime_ = 0;
        maxGumbas = 10; //Max Gumbas spawnable by a Boss
        
    }

    
    bool SOBGumbaBoss::collidesAgainst(WorldEntity* otherObject, const btCollisionShape* ownCollisionShape, btManifoldPoint& contactPoint) {

        //Every object with mass -1 does not change the direction of the GumbaBoss. For example the ground floor! The other objects switch the direction of the GumbaBoss.
        if (changeAllowed_ && otherObject->getMass() != -1) {
            goesRight_ = !goesRight_;
            changeAllowed_ = false;
        }

        return true;
    }




    void SOBGumbaBoss::tick(float dt)
    {
        SUPER(SOBGumbaBoss, tick, dt);

        if (!changeAllowed_) {
            changedOn_+= dt;
            // After a collision, we don't listen for collisions for 200ms - that's because one wall can cause several collisions!
            if (changedOn_> 0.200) {
                changeAllowed_ = true;
                changedOn_ = 0.0;

            }
        }
    
        gumbaTime_ += dt;

        if(gumbaTime_ > gumbaMaxTime_){
            spawnGumba();
            gumbaTime_ = 0;
        }

        int dir = 1;
        if (!goesRight_)
            dir = -1;

        Vector3 velocity = getVelocity();
        velocity.z -= gravityAcceleration_*dt;
        velocity.x = dir*speed_;
        setVelocity(velocity);

        lastPos_ = getPosition();

    }
    void SOBGumbaBoss::spawnGumba() {
        SOBCenterpoint* center_ = ((SOB*)getGametype())->center_;

         SOBGumba* gumba = new SOBGumba(center_->getContext());
         Vector3 spawnpos = this->getWorldPosition();
         spawnpos.z += 0;

        if (gumba != nullptr && center_ != nullptr)
        {
            gumba->addTemplate("gumbaShootable");
            bool direction = ((this->getWorldOrientation().getRoll().valueRadians())>-1.6&&(this->getWorldOrientation().getRoll().valueRadians()<1.6));
            gumba->setDirection(direction);
            if(direction)
            {
                spawnpos.x+=20;
            }
            else
            {
                spawnpos.x-=20;
            }
            spawnpos.z+=15;
            gumba->setPosition(spawnpos);

        }

        Vector3 velocity = gumba->getVelocity();
        velocity.z -= 100;
        gumba->setVelocity(velocity);
     }
}
